﻿using System.Drawing;
using NUnit.Framework;
using Rhino.Mocks;

namespace Resizer.Tests.NUnit
{
    [TestFixture]
    public class ResizeControllerTest
    {
        ImageFolders _fileNamesProvider;
        IImageIO _imageIO;
        ImageResizer _resizer;
        ScaleCalculator _scaleCalculator;
        Image _sourceImage;
        Image _resultImage;
        Size _fakeSourceSize = new Size(5000000, 5000000);
        Size _fakeDestSize = new Size(1000000, 1000000);

        [SetUp]
        public void Setup()
        {
            _fileNamesProvider = MockRepository.GenerateMock<ImageFolders>();
            _imageIO = MockRepository.GenerateMock<IImageIO>();
            _resizer = MockRepository.GenerateMock<ImageResizer>();
            _scaleCalculator = MockRepository.GenerateMock<ScaleCalculator>();
            _sourceImage = MockRepository.GenerateMock<Image>();
            _resultImage = MockRepository.GenerateMock<Image>();
        }

        private void DefaultStubs()
        {
            _imageIO.Stub(x => x.ReadImage(null)).IgnoreArguments().Return(_sourceImage);
            _fileNamesProvider.Stub(x => x.GetSourceFiles()).Return(new string[] { "a" });
            _fileNamesProvider.Stub(x => x.GetDestFileName(null)).IgnoreArguments().Return("destName");
            _scaleCalculator.Stub(x => x.Resize(default(Size))).IgnoreArguments().Return(_fakeDestSize);
            _resizer.Stub(x => x.Resize(null, default(Size))).IgnoreArguments().Return(_resultImage);
            _sourceImage.Stub(x => x.Size).Return(_fakeSourceSize);
        }

        private ResizeController CreateObject()
        {
            DefaultStubs();
            return new ResizeController(_fileNamesProvider, _imageIO, _resizer, _scaleCalculator);
        }

        [Test]
        public void DoResize_Reads_All_Source_Images()
        {
            _fileNamesProvider.Stub(x => x.GetSourceFiles()).Return(new string[] { "a", "b", "c" });
            DefaultStubs();

            CreateObject().DoResize();
            _imageIO.AssertWasCalled(x => x.ReadImage("a"));
            _imageIO.AssertWasCalled(x => x.ReadImage("b"));
            _imageIO.AssertWasCalled(x => x.ReadImage("c"));
        }

        [Test]
        public void DoResize_Calls_ScaleCalculator_With_Source_Image_Size()
        {
            CreateObject().DoResize();
            _scaleCalculator.AssertWasCalled(x => x.Resize(_fakeSourceSize));
        }

        [Test]
        public void DoResize_Calls_Resizer_With_Calculated_Size()
        {
            CreateObject().DoResize();
            _resizer.AssertWasCalled(x => x.Resize(_sourceImage, _fakeDestSize));
        }

        [Test]
        public void DoResize_Saves_Image_Returned_By_Resizer()
        {
            CreateObject().DoResize();
            _imageIO.AssertWasCalled(x=>x.SaveImage(Arg<Image>.Is.Same(_resultImage), Arg<string>.Is.Anything));
        }

        [Test]
        public void DoResize_Gets_DestFileName_For_Image()
        {
            CreateObject().DoResize();
            _fileNamesProvider.AssertWasCalled(x => x.GetDestFileName("a"));
        }

        [Test]
        public void DoResize_Saves_Image_Under_Correct_Name()
        {
            CreateObject().DoResize();
            _imageIO.AssertWasCalled(x => x.SaveImage(Arg<Image>.Is.Anything, Arg<string>.Is.Equal("destName")));
        }

    }
}
